<html>
<head>
    <title>$title</title>
    <LINK rel="Stylesheet" type="text/css" href="arch_css.css">
</head>
<body bgcolor="$bcolor">
    <table class='htable' cellpadding="0" cellspacing="0" width="100%"><td>
<td align="center" width="100%">
    <b><font size="large" color="$fcolor">$title</font></b>
</table>
</body>
</html>
